/**
 * Kode-fil for ruter-klassen
 * 
 * @file ruter.cpp
 * @author Sondre Sand & Frederik Simonsen
 */
#include <fstream>
#include <iostream>
#include <map>
#include "LesData3.h"
#include "ruter.h"
#include "rute.h"
#include "bane.h"
#include "buss.h"
#include "const.h"
#include "stoppesteder.h"
using namespace std;

extern Stoppesteder gStoppestederBase;


/**
 * ruteNrFinnes(...)
 */
bool Ruter::ruteNrFinnes(const int ruteNr) {
    return rutene.count(ruteNr);
}

/**
 * endreRute()
 */
void Ruter::endreRute() {

}

/**
 * Funksjon som håndterer ulike menyvalg fra bruker
 * 
 * @see nyRute()
 * @see skrivAlle()
 * @see endreRute()
 * @see ruteBeskrivelse()
 * @see ruteTabell()
 */
void Ruter::handling() {
    char valg;

    skrivMeny();
    valg = lesChar("\nKommando");

    switch (valg) {
        case 'N': nyRute(); break;
        case 'A': skrivAlle(); break;
        case 'E': endreRute(); break;
        case 'B': ruteBeskrivelse(); break;
        case 'T': ruteTabell(); break;
        default: cout << "\nUlovlig kommando!\n"; break;
    }
}

/**
 * Leser informasjon om ruter fra fil
 */
void Ruter::lesFraFil() {
    int key,
        antRuter;
    char ruteType;
    ifstream innfil("ruter.dta"); // Åpner aktuell fil

    if (innfil) { // Hvis filen funnet:
        cout << "Leser fra filen 'ruter.dta'...\n";
        innfil >> antRuter; innfil.ignore(); // Leser antall ruter
        innfil >> key; // Prøver å lese første port
        while (!innfil.eof()) { // Så lenge ikke slutt på fil:
            innfil >> ruteType; innfil.ignore();
            switch (ruteType) {
                case 'U': rutene[key] = new Buss(innfil); break;
                case 'A': rutene[key] = new Bane(innfil); break;
            }
            innfil >> key;
        }
        innfil.close();

    } else
        cout << "\nFant ikke filen 'ruter.dta'...\n";

}

/**
 * Lager ny rute og putter den inn der det skal
 * så lenge ruten har mer enn 1 stopp.
 * 
 * @see Ruter::ruteNrFinnes(...)
 * @see Rute::lesData()
 * @see Rute::erListeGyldig()
 * @see Ruter::ruteBeskrivelse
 * @see Rute::slettData()
 */
void Ruter::nyRute() {
    int ruteNr;
    char ruteType;
    Rute* nyRute = nullptr;

    if (!gStoppestederBase.tom()) { // Hvis det finnes stoppesteder registrert:
        ruteNr = lesInt("Rutenr", MINRUTER, MAXRUTER);
        // Hvis ikke duplikat rutenr blir skrevet inn:
        if (!ruteNrFinnes(ruteNr)) {

            ruteType = lesChar("Rutetype b(A)ne eller b(U)ss");
            // Looper til lovlig valg av rutetype blir lest inn:
            while (ruteType != 'A' && ruteType != 'U') {
                cout << "\nUgyldig valg, prøv igjen.\n";
                ruteType = lesChar("Rutetype b(A)ne eller b(U)ss");
            }

            switch (ruteType) {
                case 'A': nyRute = new Bane; nyRute->lesData(); break;
                case 'U': nyRute = new Buss; nyRute->lesData(); break;
            }

            if (nyRute->erListeGyldig()){
                rutene[ruteNr] = nyRute;
                cout << "\nNy rute er lagt til:\n";
                ruteBeskrivelse(ruteNr);
            } else nyRute->slettData();

        } else // Rutenummer er duplikat:
            cout << "\nRutenummeret finnes fra før. Kan ikke legge til rute.\n\n";

    } else // Hvis ingen stoppesteder registrert:
        cout << "\nIngen stopper registrert i datastrukturen, kan derfor ikke "
             << "opprette ny rute for øyeblikket.\n\n";
}

/**
 * Ber om et rutenummer, skriver ut gitt rute
 * 
 * @see Rute::skrivBeskrivelse()
 */
void Ruter::ruteBeskrivelse() {
    int ruteNr;
    ruteNr = lesInt("\nRutenr", MINRUTER, MAXRUTER);
    auto it = rutene.find(ruteNr);
    if (it != rutene.end()) {
        it->second->skrivBeskrivelse();
    } else
        cout << "\nRuten med dette rutenr eksisterer ikke..\n";
}

/**
 * skriver ut rute fra med sent nummer.
 * brukes for å skrive rute når den først blir laget.
 * 
 * @param ruteNr -rute som skal skrives ut 
 * @see Rute::skrivBeskrivelse
 */
void Ruter::ruteBeskrivelse(const int ruteNr){
    auto it = rutene.find(ruteNr);
    it->second->skrivBeskrivelse();
}

/**
 * ruteTabell()
 */
void Ruter::ruteTabell() {
    int ruteNr;
    char valg;
    string navn;
    //Rute* ruten = nullptr;

    if (!rutene.empty()) { // Hvis det er registrert ruter:
        ruteNr = lesInt("Rutenr", MINRUTER, MAXRUTER);
        if (ruteNrFinnes(ruteNr)) { // Hvis rutenummeret finnes:
            auto ruten = rutene.find(ruteNr); // Iterator til riktig rute
            ruten->second->skrivRetninger(); // Skriver kort info om ruten
            valg = lesChar("Rutetabell for retningen (F/T)");
            // Looper til lovlig valg av retning:
            while (valg != 'F' && valg != 'T') {
                cout << "\nUgyldig valg, prøv igjen.\n";
                valg = lesChar("Rutetabell for retningen (F/T)");
            }

            ruten->second->skrivKort(); // Skriver info om rutetypen
            
            switch (valg) { // Skriver hele ruten:
                case 'F': ruten->second->skrivRute(Fram); break;
                case 'T': ruten->second->skrivRute(Tilbake); break;
            }

            /*cout << "\n\nRutetabell for stoppestedet (navn): ";
            getline(cin, navn);*/


        } else // Hvis rutenummeret ikke finnes:
            cout << "\nDette rutenummeret er ikke registrert.\n\n";
    } else // Hvis ingen ruter registrert:
        cout << "\nIngen ruter registrert i programmet.\n\n";
}

/**
 * skriver alle ruter som finnes.
 * 
 * @see Rute::skrivData
 */
void Ruter::skrivAlle() {
    cout << "\nFølgende ruter finnes:\n";
    for (const auto & [ruteNr, rute] : rutene) {
        cout << ruteNr << ' ';
        rute->skrivData();
        cout << '\n';
    }
    cout << '\n';
}

/**
 * skriver menyen
 */
void Ruter::skrivMeny() {
    cout << "\nFølgende kommandoer er tilgjengelige:\n"
         << "\tN - Ny rute\n"
         << "\tA - Skriv alle ruter\n"
         << "\tE - Endre en rute\n"
         << "\tB - Beskrivelse av en rute\n"
         << "\tT - Rutetabell for et stoppested\n";
}

/**
 * Skriver informasjon om ruter til fil
 * 
 * @see Rute::skrivTilFil(...)
 */
void Ruter::skrivTilFil() {
    ofstream utfil("ruter.dt2"); // Åpner aktuell fil for skriving
    cout << "\nSkriver til filen 'ruter.dt2'...\n";

    utfil << rutene.size() << '\n';
    for (const auto & val : rutene) {
        utfil << val.first << ' ';
        (val.second)->skrivTilFil(utfil);
    }
    utfil.close();
}

